class os():
    @classmethod
    def mkdir(self, path, inquirer=False, inquirer_message=None, inquirer_color="blue", verbose=False):
        import os
        if inquirer_message is None:
            inquirer_message = "Create directory: %s? [Y/n] " % path
        if inquirer:
            from avania.sys.prompt import prompt
            if not prompt.prompt(message=inquirer_message, default=True, color=inquirer_color):
                return False
        try:
            os.makedirs(path, exist_ok=True)
            if verbose:
                prompt.print("Directory created: %s" % path, "green")
            return True
        except:
            if verbose:
                prompt.print("Failed to create directory: %s" % path, "red")
            return False

    @classmethod
    def create_file(self, path, content):
        import os
        from avania.sys.prompt import prompt
        prompt.print("test")     

    @classmethod
    def hasdir(self, path, verbose=False):
        try:
            import os
            return os.path.isdir(path)
        except:
            from avania.sys.prompt import prompt
            if verbose:
                prompt.print("Failed to check if path is directory: %s" % path, color="red")
            return False

    @classmethod
    def listdir(self, path, verbose=False, fail_message=None):
        if fail_message is None:
            fail_message = "Failed to list directory: %s" % path
        try:
            import os
            return os.listdir(path)
        except:
            from avania.sys.prompt import prompt
            if verbose:
                prompt.print(fail_message % path, "red")
            return []
        
    @classmethod
    def exists(self, path, verbose=False):
        try:
            import os
            return os.path.exists(path)
        except:
            from avania.sys.prompt import prompt
            if verbose:
                prompt.print("Failed to check if path exists: %s" % path, "red")
            return False
        
    @classmethod
    def cp(self, copied_file, new_file, verbose=False, inquirer=False, inquirer_message=None, inquirer_color="blue",
           overwrite=False, success_message=None, fail_message=None):
        from avania.sys.prompt import prompt
        if not self.exists(copied_file, verbose=verbose):
            if verbose:
                prompt.print("File %s does not exist" % copied_file, "red")
                prompt.print_github()
            return False
        if success_message is None:
            success_message = "Copied %s to %s" % (copied_file, new_file)
        if fail_message is None:
            fail_message = "Failed to copy %s to %s" % (copied_file, new_file)
        if not overwrite and self.exists(new_file, verbose=verbose):
            if verbose:
                prompt.print("File %s already exists" % new_file, "yellow")
            return False
        if inquirer_message is None:
            inquirer_message = "Copy %s to %s? [Y/n] " % (copied_file, new_file)
        try:
            import os
            from avania.sys.prompt import prompt
            if inquirer:
                if not prompt.prompt(inquirer_message, default=True, color=inquirer_color):
                    return False
            with open(copied_file) as f:
                with open(new_file, 'w') as new_file:
                    new_file.write(f.read())
                    if verbose:
                        prompt.print(success_message, "green")
                    return True
        except:
            from avania.sys.prompt import prompt
            if verbose:
                prompt.print(fail_message, "red")
            return False
    
    @classmethod
    def read(self, path, verbose=False, fail_message=None):
        if fail_message is None:
            fail_message = "Failed to read file: %s" % path
        try:
            if not self.exists(path, verbose=verbose):
                return False
            with open(path) as f:
                return f.read()
        except:
            from avania.sys.prompt import prompt
            if verbose:
                prompt.print(fail_message, "red")
            return False
        
    @classmethod
    def avania__path__(self):
        try:
            import os
            return os.path.abspath(os.path.join(os.path.dirname(__file__), '../../..' + '/avania'))
        except:
            return f"./avania"


import mysql.connector

class database():
    def __init__(self, config: dict, database: str = None):
        # 单独获取config中的database键值对
        self.database_name = config['database']
        # 连接mysql，但是不连接到任何数据库，注意config中有database键值对，所以要删除
        del config['database']
        self.connection = mysql.connector.connect(**config)
        # 创建一个游标
        self.cursor = self.connection.cursor()
        # 检查是否存在数据库，如果不存在则创建数据库
        self.cursor.execute('SHOW DATABASES')
        databases = self.cursor.fetchall()
        databases = [database[0] for database in databases]
        if self.database_name not in databases:
            self.cursor.execute('CREATE DATABASE {}'.format(self.database_name))
        # 关闭游标
        self.cursor.close()
        # 关闭连接  
        self.connection.close()
        # 销毁class
        del self


import mysql.connector
from .mate import MysqliMetaClass

class mysqli(metaclass=MysqliMetaClass):
    config = None
    connection = None

    def __get_connection__(self):
            try:
                connection = mysql.connector.connect(**self.config)
                return connection
            except mysql.connector.Error as e:
                print('Error: Connection failed')
                print(e)
            return None
    
    def __connect__(self):
        if self.connection is None:
            self.connection = self.__get_connection__()
        return self.connection
    
    def __call__(self, sql: str, values: dict = None, type: str = 'fetchall'):
        # 检查values是否为None，如果是None则将values设置为一个空元组
        if values == None:
            values = ()
        try:
            cursor = self.connection.cursor()
            cursor.execute(sql, values)
            self.connection.commit()
            if type == 'fetchall':
                return cursor.fetchall()
            elif type == 'fetchone':
                return cursor.fetchone()
            elif type == 'fetchmany':
                return cursor.fetchmany()
            elif type == 'execute':
                return cursor.rowcount
            elif type == 'lastrowid':
                return cursor.lastrowid
            cursor.close()
        except Exception as e:
            print('Error: Query failed')
            print(e)
        return self

    @classmethod
    def insert(self, table: str, keys: list, values: list):
        print("111"*111)
        sql = 'INSERT INTO {} ({}) VALUES ({})'.format(table, ', '.join(keys), ', '.join(['%s' for key in keys]))
        print(sql)
        return self(sql, values, 'execute')
    
    def print(self, a):
        print(a, self.d)
    
    def update(self, table: str, keys: list, values: list, where: str):
        sql = 'UPDATE {} SET {} WHERE {}'.format(table, ', '.join([key + ' = %s' for key in keys]), where)
        return self(sql, values, 'execute')
    
    def delete(self, table: str, where: str):
        sql = 'DELETE FROM {} WHERE {}'.format(table, where)
        return self(sql, type = 'execute')
    
    def select(self, table: str, keys: list, where: str = None, values: dict = None, type: str = 'fetchall'):
        sql = 'SELECT {} FROM {}'.format(', '.join(keys), table)
        if where is not None:
            sql += ' WHERE {}'.format(where)
        return self(sql, values, type)
    
    def find(self, table: str, keys: list, where: str = None, values: dict = None):
        return self.select(table, keys, where, values, 'fetchone')
    
    def __del__(self):
        # self.connection.close()
        print('Connection closed')
        del self



from .database import database

class MysqliMetaClass(type):
    __config__ = None
    __database_name__ = None
    __has_database__ = False

    def __new__(cls, name, bases, attrs):
        print(cls)
        # 如果__config__和__database_name__为None，则获取config和database_name
        if cls.__config__ is None and cls.__database_name__ is None:
            attrs['config'], attrs['database_name'] = cls.__get_config__()
        if attrs['has_database'] is False:
            pass
        return type.__new__(cls, name, bases, attrs)


    @staticmethod
    def __get_config__():
        try:
            from config.database import mysql_host, mysql_user, mysql_password, mysql_port
            return {
                'host': mysql_host,
                'user': mysql_user,
                'password': mysql_password,
                'port': mysql_port,
            }
        except:
            # 如果 config/database.py 不存在，则提示
            print('Error: Config file not found')
            return {
                'host': 'localhost',
                'user': 'root',
                'password': '',
                'port': 3306,
            }
        
    @staticmethod
    def __get_database_name__():
        try:
            from config.database import mysql_database
            return mysql_database
        except:
            # 如果 config/database.py 不存在，则提示
            print('Error: Config file not found')
            return 'avania'
    
    


class prompt():
    colors = {
        "red": "\033[31m",
        "green": "\033[32m",
        "yellow": "\033[33m",
        "blue": "\033[34m",
        "purple": "\033[35m",
        "cyan": "\033[36m",
        "white": "\033[37m",
        "reset": "\033[0m"
    }

    @classmethod
    def print(self, message, color = 'white'):
        if color in self.colors:
            print("%s%s%s" % (self.colors[color], message, self.colors["reset"]))
        else:
            print(message)

    @classmethod
    def prompt(self, message, default = False, color = "blue"):
        if color in self.colors:
            response = input("%s%s%s" % (self.colors[color], message, self.colors["reset"]))
        else:
            response = input(message)
        if response == "":
            return default
        elif response.lower() in ["y", "yes"]:
            return True
        elif response.lower() in ["n", "no"]:
            return False
        else:
            return prompt(message, default, color)
        
    @classmethod
    def print_github(self):
        prompt.print("Please visit https://github.com/StarLxa/avania-core for more information", "green")


import os
from .meta import ConfigMetaClass

class config(metaclass = ConfigMetaClass):
    @classmethod
    def get(self, name, attr=False):
        if attr:
            return self.get_attr(name, attr)
        else:
            return self.parse_file(self.get_file(name))



from setuptools import setup, find_packages

setup(
    name='avania',
    version='0.1.7',
    description='Pays tribute to Laravel',
    long_description_content_type='text/markdown',
    long_description=open('CONTRIBUTING.md').read(),
    author='Simon',
    packages=find_packages(),
    install_requires=[
        'mysql-connector-python',
        'flask'
    ],
)



class ConfigMetaClass(type):
    path = 'config'
    template_base_path = '/'
    template_path = '/sys/config/templates'
    first = False

    def __new__(cls, name, bases, attrs):
        cls.update_template_base_path()
        cls.check_folder()
        return type.__new__(cls, name, bases, attrs)
    
    @classmethod
    def update_template_base_path(self):
        from avania.sys.os import os
        self.template_base_path = os.avania__path__()
        return self

    @classmethod
    def check_folder(self):
        from avania.sys.os import os
        if not os.hasdir(self.path):
            self.first = True
            self.create_folder()
        return self
    
    @classmethod
    def create_folder(self):
        from avania.sys.os import os
        if os.mkdir(self.path, inquirer=True, inquirer_message="Create config folder? [Y/n] ", verbose=True):
            self.create_templates()
        

    @classmethod
    def create_templates(self):
        from avania.sys.prompt import prompt
        if prompt.prompt("Create all template files? [Y/n] ", default=True):
            from avania.sys.os import os
            # 从avania/config/templates中复制所有文件到config文件夹
            for file in os.listdir(f'{self.template_base_path}{self.template_path}'):
                # 如果不是__init__.py文件
                if file != '__init__.py':
                    # 创建文件
                    self.create_template(file.replace('.py', ''), inquirer=False)
        
    @classmethod
    def create_template(self, name, inquirer = True, verbose = True):
        from avania.sys.os import os
        from avania.sys.os.config import git_url
        from avania.sys.prompt import prompt
        if self.first and inquirer:
            prompt.print(f'Failed to find {name}.py in config folder', 'red')
            return False
        return os.cp(f'{self.template_base_path}{self.template_path}/{name}.py', f'config/{name}.py', verbose=verbose, 
                     inquirer=inquirer, inquirer_message=f'Create {name}.json? [Y/n] ',
                     success_message=f'Created {name}.py', fail_message=f'Failed to create {name}.py')

    @classmethod
    def get_file(self, name):
        from avania.sys.os import os
        result = os.read(f'{self.path}/{name}.py')
        if not result:
            if self.create_template(name):
                return self.get_file(name)
            else:
                return None
        else:
            return result

    # 解析python文件的内容，返回一个JSON对象
    # 文件内容为xxx=xxx
    # 返回{"xxx": "xxx"}
    @classmethod
    def parse_file(self, text):
        result = {}
        try:
            for line in text.split('\n'):
                if line:
                    key, value = line.split('=')
                    # 去掉两边的空格
                    key, value = key.strip(), value.strip()
                    # 如果value为True或False，转换为bool类型
                    if value.lower() == 'true':
                        value = True
                    elif value.lower() == 'false':
                        value = False
                    # 否则如果value为字符串，去掉两边的引号
                    elif value[0] == value[-1] == "'" or value[0] == value[-1] == '"':
                        value = value[1:-1]
                    # 否则如果value为数字，转换为int类型
                    else:
                        try:
                            value = int(value)
                        except:
                            pass
                    result[key] = value
        except:
            return None
        return result
        


    @classmethod
    def get_attr(self, name, attr):
        datum = self.parse_file(self.get_file(name))
        if datum is None:
            return None
        return datum.get(attr, None)
        

            
       